from typing import List, Literal
from .api_base_model import ApiBaseModelWithIdNameLabelAndDesc
from .quantity import Quantity
from .administration_duration import AdministrationDuration
from .alias_code import AliasCode
from .comment_annotation import CommentAnnotation

class AgentAdministration(ApiBaseModelWithIdNameLabelAndDesc):
  duration:	AdministrationDuration
  dose:	Quantity
  route:	AliasCode
  frequency:	AliasCode
  notes: List[CommentAnnotation] = []
  instanceType: Literal['AgentAdministration']



from typing import Literal, Union
from .api_base_model import ApiBaseModelWithIdNameLabelAndDesc
from .geographic_scope import SubjectEnrollment

class StudySite(ApiBaseModelWithIdNameLabelAndDesc):
  currentEnrollment: Union[SubjectEnrollment, None] = None
  instanceType: Literal['StudySite']



from typing import Literal, Union
from .api_base_model import ApiBaseModelWithId
from .code import Code
from .alias_code import AliasCode
from .quantity import Quantity

class GeographicScope(ApiBaseModelWithId):
  type: Code
  code: Union[AliasCode, None] = None
  instanceType: Literal['GeographicScope']

class SubjectEnrollment(GeographicScope):
  quantity: Quantity
  instanceType: Literal['SubjectEnrollment']



import pandas as pd
from usdm_excel.study_design_sites_sheet.study_design_sites_sheet import StudyDesignSitesSheet
from usdm_model.code import Code


def test_create(mocker, globals):
  globals.cross_references.clear()
  mock_present = mocker.patch("usdm_excel.base_sheet.BaseSheet._sheet_present")
  mock_present.side_effect=[True]
  mock_id = mocker.patch("usdm_excel.id_manager.IdManager.build_id")
  mock_id.side_effect=['Site_1', 'Org_1', 'Addr_1', 'Id_1', 'Site_2', 'Org_2', 'Addr_2', 'Id_2', 'Site_3', 'Org_3', 'Addr_3', 'Id_3']
  mocked_open = mocker.mock_open(read_data="File")
  mocker.patch("builtins.open", mocked_open)
  data = [
    [ 'CRO1', 'CRO One',   'Study Registry', 'DUNS', '111111111', 'line|district|city|state|postal_code|GBR',      'Site1', 'Site One', 'Big Site' ],
    [ 'CRO2', 'CRO Two',   'Study Registry', 'DUNS', '222222222', 'line2,district2,city2,state2,postal_code2,FRA', 'Site2', 'Site Two', 'Little Site'],
    [ 'CRO3', 'CRO Three', 'Study Registry', 'DUNS', '333333333', 'line3,district3,city3,state3,postal_code3,FR',  'Site3', 'Site Three', 'Middle Site']
  ]
  mock_read = mocker.patch("pandas.read_excel")
  mock_read.return_value = pd.DataFrame(data, columns=['name', 'label', 'type', 'identifierScheme', 'identifier', 'address', 'siteName', 'siteLabel', 'siteDescription'])
  mock_json = mocker.patch("json.load")
  mock_json.return_value = {}
  expected_1 = Code(id='Code1', code='code', codeSystem='codesys', codeSystemVersion='3', decode="GBR")
  expected_2 = Code(id='Code2', code='code', codeSystem='codesys', codeSystemVersion='3', decode="FRA")
  expected_3 = Code(id='Code3', code='code', codeSystem='codesys', codeSystemVersion='3', decode="FRA")
  mock_code = mocker.patch("usdm_excel.iso_3166.ISO3166.code")
  mock_code.side_effect=[expected_1, expected_2, expected_3]
  sites = StudyDesignSitesSheet("", globals)
  assert len(sites.organizations) == 3
  assert len(sites.sites) == 3
  assert sites.organizations[0].id == 'Id_1'
  assert sites.organizations[0].identifier == '111111111'
  assert sites.organizations[0].identifierScheme == 'DUNS'
  assert sites.organizations[0].legalAddress.city == 'city'
  assert sites.organizations[0].legalAddress.country.decode == 'GBR'
  assert sites.organizations[0].manages[0].id == 'Site_1'
  assert sites.organizations[1].id == 'Id_2'
  assert sites.organizations[1].identifier == '222222222'
  assert sites.organizations[1].identifierScheme == 'DUNS'
  assert sites.organizations[1].legalAddress.city == 'city2'
  assert sites.organizations[2].id == 'Id_3'
  assert sites.organizations[2].identifier == '333333333'
  assert sites.organizations[2].manages[0].id == 'Site_3'
  
def test_create_empty(mocker, globals):
  globals.cross_references.clear()
  mock_present = mocker.patch("usdm_excel.base_sheet.BaseSheet._sheet_present")
  mock_present.side_effect=[True]
  mocked_open = mocker.mock_open(read_data="File")
  mocker.patch("builtins.open", mocked_open)
  data = []
  mock_read = mocker.patch("pandas.read_excel")
  mock_read.return_value = pd.DataFrame(data, columns=['studyIdentifierName', 'studyIdentifierDescription', 'studyIdentifierType'])
  sites = StudyDesignSitesSheet("", globals)
  assert len(sites.organizations) == 0

def test_read_cell_by_name_error(mocker, globals):
  globals.cross_references.clear()
  mock_present = mocker.patch("usdm_excel.base_sheet.BaseSheet._sheet_present")
  mock_present.side_effect=[True]
  mock_error = mocker.patch("usdm_excel.errors_and_logging.errors.Errors.add")
  mocked_open = mocker.mock_open(read_data="File")
  mocker.patch("builtins.open", mocked_open)
  data = [
    [ 'CRO1', 'CRO One', 'DUNS', '111111111', 'line|district|city|state|postal_code|GBR',      'Site1', 'Site One', 'Big Site' ],
  ]
  mock_read = mocker.patch("pandas.read_excel")
  mock_read.return_value = pd.DataFrame(data, columns=['name', 'label', 'identifierScheme', 'identifier', 'address', 'siteName', 'siteLabel', 'siteDescription'])
  sites = StudyDesignSitesSheet("", globals)
  mock_error.assert_called()
  assert mock_error.call_args[0][0] == "studyDesignSites"
  assert mock_error.call_args[0][1] == None
  assert mock_error.call_args[0][2] == None
  assert mock_error.call_args[0][3] == "Exception. Error [Failed to detect column(s) 'type' in sheet] while reading sheet 'studyDesignSites'. See log for additional details."



from typing import Literal
from .api_base_model import ApiBaseModelWithId
from .organization import Organization

class StudyIdentifier(ApiBaseModelWithId):
  studyIdentifier: str
  studyIdentifierScope: Organization
  instanceType: Literal['StudyIdentifier']



import traceback
from usdm_model.organization import Organization
from usdm_model.study_identifier import StudyIdentifier
from usdm_excel.base_sheet import BaseSheet
from usdm_excel.globals import Globals


class StudyIdentifiersSheet(BaseSheet):

  SHEET_NAME = 'studyIdentifiers'
  
  def __init__(self, file_path, globals: Globals):
    try:
      self.identifiers = []
      super().__init__(file_path=file_path, globals=globals, sheet_name=self.SHEET_NAME)
      self.process_sheet()
    except Exception as e:
      self._sheet_exception(e)
      
  def process_sheet(self):
    self.identifiers = []
    for index, row in self.sheet.iterrows():
      org_type = self.read_cdisc_klass_attribute_cell_by_name('Organization', 'organizationType', index, ['organisationType', 'organizationType', 'type'])     
      org_id_scheme = self.read_cell_by_name(index, ['organisationIdentifierScheme', 'organizationIdentifierScheme', 'identifierScheme'])
      org_identifier = self.read_cell_by_name(index, ['organisationIdentifier', 'organizationIdentifier'])
      org_name = self.read_cell_by_name(index, ['organisationName', 'organizationName', 'name'])
      org_label = self.read_cell_by_name(index, 'label', default="", must_be_present=False)
      org_address = self.read_address_cell_by_name(index, ['organisationAddress', 'organizationAddress', 'address'])
      if org_address:
        self.globals.cross_references.add(org_address.id, org_address)   
      organisation = self.create_object(Organization, {'identifierScheme': org_id_scheme, 'identifier': org_identifier, 'name': org_name, 'label': org_label, 'organizationType': org_type, 'legalAddress': org_address})
      if organisation:
        self.globals.cross_references.add(organisation.id, organisation)   
        item = self.create_object(StudyIdentifier, {'studyIdentifier': self.read_cell_by_name(index, 'studyIdentifier'), 'studyIdentifierScope': organisation})
        if item:
          self.identifiers.append(item)
          self.globals.cross_references.add(item.studyIdentifier, item)         



import traceback
from usdm_excel.base_sheet import BaseSheet
from usdm_excel.alias import Alias
from usdm_model.study_intervention import StudyIntervention
from usdm_model.agent_administration import AgentAdministration
from usdm_model.administration_duration import AdministrationDuration
from usdm_excel.globals import Globals

class StudyDesignInterventionSheet(BaseSheet):

  SHEET_NAME = 'studyDesignInterventions'

  def __init__(self, file_path: str, globals: Globals):
    try:
      self.items = []
      super().__init__(file_path=file_path, globals=globals, sheet_name=self.SHEET_NAME)
      self.current_name = None
      self.current_intervention = None
      for index, row in self.sheet.iterrows():
        # Read the adminstrations and durations
        admin_duration = self._create_administration_duration(index)
        agent_admin = self._create_agent_administration(index, admin_duration)
        # Read intervention in present
        self._create_intervention(index, agent_admin)
    except Exception as e:
      self._sheet_exception(e)

  def _create_intervention(self, index, agent_admin):
    name = self.read_cell_by_name(index, 'name')
    if name and name != self.current_name:
      self.current_name = name
      description = self.read_cell_by_name(index, 'description', must_be_present=False)
      label = self.read_cell_by_name(index, 'label', must_be_present=False)
      codes = self.read_other_code_cell_multiple_by_name(index, "codes")
      role = self.read_cdisc_klass_attribute_cell_by_name("StudyIntervention", "role", index, "role")
      type = self.read_cdisc_klass_attribute_cell_by_name("StudyIntervention", "type", index, "type")
      pharm_class = self.read_other_code_cell_by_name(index, "pharmacologicalClass")
      product_designation = self.read_cdisc_klass_attribute_cell_by_name("StudyIntervention", "productDesignation", index, "productDesignation")
      min_duration =self.read_quantity_cell_by_name(index, "minimumResponseDuration")
      self.current_intervention = self._intervention(name, description, label, role, codes, type, pharm_class, product_designation, min_duration, agent_admin)
    else:
      self.current_intervention.administrations.append(agent_admin)

  def _intervention(self, name, description, label, role, codes, type, pharm_class, product_designation, minimum_duration, agent_administration):
    try:
      item = StudyIntervention(
        id=self.globals.id_manager.build_id(StudyIntervention), 
        name=name, 
        description=description, 
        label=label,
        role=role,
        type=type,
        minimumResponseDuration=minimum_duration,
        codes=codes,
        productDesignation=product_designation,
        pharmacologicClass=pharm_class,
        administrations=[agent_administration]
      )
    except Exception as e:
      self._general_error(f"Failed to create StudyIntervention object", e)
    else:
      self.items.append(item)
      self.globals.cross_references.add(name, item)
      return item    

  def _create_agent_administration(self, index, admin_duration):
    name = self.read_cell_by_name(index, 'administrationName')
    descriptiopn = self.read_cell_by_name(index, 'administrationDescription', must_be_present=False)
    label = self.read_cell_by_name(index, 'administrationLabel', must_be_present=False)
    route = self.read_cdisc_klass_attribute_cell_by_name("AgentAdministration", "route", index, "administrationRoute")
    dose = self.read_quantity_cell_by_name(index, "administrationDose")
    frequency = self.read_cdisc_klass_attribute_cell_by_name("AgentAdministration", "frequency", index, "administrationFrequency")
    return self._agent_administration(name, descriptiopn, label, route, dose, frequency, admin_duration)

  def _agent_administration(self, name, description, label, route, dose, frequency, admin_duration):
    try:
      item = AgentAdministration(
        id=self.globals.id_manager.build_id(AgentAdministration), 
        name=name, 
        description=description, 
        label=label,
        duration=admin_duration,
        dose=dose,
        route=Alias(self.globals).code(route, []),
        frequency=Alias(self.globals).code(frequency, [])
      )
    except Exception as e:
      self._general_error(f"Failed to create AgentAdministration object", e)
    else:
      self.globals.cross_references.add(name, item)
      return item

  def _create_administration_duration(self, index):
    description = self.read_cell_by_name(index, 'administrationDurationDescription', must_be_present=False)
    will_vary = self.read_boolean_cell_by_name(index, 'administrationDurationWillVary')
    will_vary_reason = self.read_cell_by_name(index, 'administrationDurationWillVaryReason')
    quantity = self.read_quantity_cell_by_name(index, 'administrationDurationQuantity')
    return self._administration_duration(description, will_vary, will_vary_reason, quantity)

  def _administration_duration(self, description, will_vary, will_vary_reason, quantity):
    try:
      item = AdministrationDuration(
        id=self.globals.id_manager.build_id(AdministrationDuration), 
        description=description,
        durationWillVary=will_vary,
        reasonDurationWillVary=will_vary_reason,
        quantity=quantity
      )
    except Exception as e:
      self._general_error(f"Failed to create AdministrationDuration object", e)
    else:
      return item



from typing import List, Literal, Union
from .api_base_model import ApiBaseModelWithIdNameAndLabel
from .code import Code
from .address import Address
from .study_site import StudySite

class Organization(ApiBaseModelWithIdNameAndLabel):
  organizationType: Code
  identifierScheme: str
  identifier: str
  legalAddress: Union[Address, None] = None
  instanceType: Literal['Organization']

class ResearchOrganization(Organization):
  manages: List[StudySite]
  instanceType: Literal['ResearchOrganization']
